__authors__ = ""
__copyright__ = "(c) 2014, pymal"
__license__ = "BSD License"
__contact__ = "Name Of Current Guardian of this file <email@address>"

__all__ = ['Account', 'Anime', 'Manga', 'Seasons']


def get_version():
    return '0.5b2'
